@@ -68,8 +68,8 @@ class BigPageTableMagic(obj.ProfileModification):
             (6, 2, '32bit') : [[92, 88]],
             (6, 2, '64bit') : [[-5200, -5224]], 
             (6, 3, '32bit') : [[116, 120]],
-            (6, 4, '64bit') : [[208, 184], [168, 192], [176, 168], [48, 40], [32, 24], [24, 48]],
-            (6, 4, '32bit') : [[-168, -164]],
+            (6, 4, '64bit') : [[208, 184], [168, 192], [176, 168], [48, 40], [32, 24], [24, 48], [56, 32], [-56, -10328], [24, 32], [-10344, -10336], [-10328, -10288], [-48, -10344], [-5208, -5200], [-188, -200], [40, 32], [-5200, -5208], [64, 24], [-10328, -10320], [32, 40], [-56, -64], [-10312, -10320], [24, 64], [-10304, -10344], [-64, -72], [-10328, -10336], [40, 48], [10304, 10296], [10304, 16], [-5192, -5184], [10320, 10312], [-64, -56], [-40, -64], [-10320, -10344], [-48, -72], [-72, -64], [-10304, -10328], [-56, -48]],
+            (6, 4, '32bit') : [[-168, -164], [-160, -172]],
         }
 
         version = (m.get('major', 0), m.get('minor', 0), m.get('memory_model', '32bit'))
@@ -78,7 +78,7 @@ class BigPageTableMagic(obj.ProfileModification):
         if distance == None:
             if version == (6, 3, '64bit'):
                 if m.get('build', 0) == 9601:
-                    distance = [[-5192, -5200], [-5224, -5232]]
+                    distance = [[-5192, -5200], [-5224, -5232], [-5192, -5216]]
                 else:
                     distance = [[-5200, -5176], [-5224, -5232], [-5192, -5200]]
 
@@ -116,8 +116,12 @@ class BigPageTable(obj.VolatilityMagic):
             table_size = obj.Object("address", 
                 offset = track_table - pair[1], 
                 vm = self.obj_vm)
-
-            if table_size != 0 and self.obj_vm.is_valid_address(table_base):
+                
+            if (table_base % 0x1000 == 0 and
+                    self.obj_vm.is_valid_address(table_base) and
+                    table_size != 0 and 
+                    table_size % 0x1000 == 0 and 
+                    table_size < 0x1000000):
                 break
 
         debug.debug("Distance Map: {0}".format(repr(self.distance)))
