@@ -23,7 +23,9 @@
 
 import os
 import re
+from volatility import renderers
 import volatility.plugins.procdump as procdump
+from volatility.renderers.basic import Address
 import volatility.win32.tasks as tasks
 import volatility.debug as debug
 import volatility.utils as utils
@@ -52,6 +54,11 @@ class DLLDump(procdump.ProcDump):
     def calculate(self):
         addr_space = utils.load_as(self._config)
 
+        if self._config.DUMP_DIR == None:
+            debug.error("Please specify a dump directory (--dump-dir)")
+        if not os.path.isdir(self._config.DUMP_DIR):
+            debug.error(self._config.DUMP_DIR + " is not a directory")
+
         if self._config.OFFSET != None:
             data = [self.virtual_process_from_physical_offset(addr_space, self._config.OFFSET)]
         else:
@@ -86,6 +93,30 @@ class DLLDump(procdump.ProcDump):
                             continue
                     yield proc, ps_ad, mod.DllBase.v(), mod.BaseDllName
 
+    def generator(self, data):
+        for proc, ps_ad, mod_base, mod_name in data:
+            if not ps_ad.is_valid_address(mod_base):
+                result = "Error: DllBase is unavailable (possibly due to paging)"
+            else:
+                process_offset = ps_ad.vtop(proc.obj_offset)
+                dump_file = "module.{0}.{1:x}.{2:x}.dll".format(proc.UniqueProcessId, process_offset, mod_base)
+                result = self.dump_pe(ps_ad, mod_base, dump_file)
+            yield (0,
+                   [Address(proc.obj_offset),
+                    str(proc.ImageFileName),
+                    Address(mod_base),
+                    str(mod_name or ''),
+                    str(result)])
+
+    def unified_output(self, data):
+
+        return renderers.TreeGrid(
+                          [("Process(V)", Address),
+                           ("Name", str),
+                           ("Module Base", Address),
+                           ("Module Name", str),
+                           ("Result", str)], self.generator(data))
+
     def render_text(self, outfd, data):
         if self._config.DUMP_DIR == None:
             debug.error("Please specify a dump directory (--dump-dir)")
@@ -101,7 +132,7 @@ class DLLDump(procdump.ProcDump):
 
         for proc, ps_ad, mod_base, mod_name in data:
             if not ps_ad.is_valid_address(mod_base):
-                result = "Error: DllBase is unavailable (possibly due to paging)"
+                result = "Error: DllBase is paged"
             else:
                 process_offset = ps_ad.vtop(proc.obj_offset)
                 dump_file = "module.{0}.{1:x}.{2:x}.dll".format(proc.UniqueProcessId, process_offset, mod_base)
