@@ -175,9 +175,10 @@ class DeviceTree(filescan.DriverScan):
 
     def render_text(self, outfd, data):
         for driver in data:
+            header = driver.get_object_header()
 
             outfd.write("DRV 0x{0:08x} {1}\n".format(driver.obj_offset,
-                str(driver.DriverName or '')))
+                str(driver.DriverName or header.NameInfo.Name or '')))
 
             for device in driver.devices():
 
