@@ -5,10 +5,9 @@
 # This file is part of Volatility.
 #
 # Volatility is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License Version 2 as
-# published by the Free Software Foundation.  You may not use, modify or
-# distribute this program under any other version of the GNU General
-# Public License.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # Volatility is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,6 +29,8 @@ import volatility.plugins.taskmods as taskmods
 import volatility.plugins.modscan as modscan
 import volatility.plugins.malware.malfind as malfind
 import volatility.debug as debug
+from volatility.renderers import TreeGrid
+from volatility.renderers.basic import Address, Hex
 
 try:
     import distorm3 #pylint: disable-msg=W0611
@@ -333,18 +334,18 @@ class Threads(taskmods.DllList):
         system_range = tasks.get_kdbg(addr_space).MmSystemRangeStart.dereference_as("Pointer")
 
         # Only show threads owned by particular processes
+        pidlist = []
         if self._config.PID:
             pidlist = [int(p) for p in self._config.PID.split(',')]
-        else:
-            pidlist = []
+        elif self._config.OFFSET:
+            process = self.virtual_process_from_physical_offset(addr_space, self._config.OFFSET)
+            if process:
+                pidlist = [int(process.UniqueProcessId)]
 
         # Get sorted list of kernel modules 
         mods = dict((addr_space.address_mask(mod.DllBase), mod) for mod in modules.lsmod(addr_space))
         mod_addrs = sorted(mods.keys())
 
-        # Gather processes 
-        all_tasks = list(tasks.pslist(addr_space))
-
         # Are we on x86 or x64. Save this for render_text 
         self.bits32 = addr_space.profile.metadata.\
             get("memory_model", "32bit") == "32bit"
@@ -362,7 +363,7 @@ class Threads(taskmods.DllList):
         seen_threads = dict()
 
         # Gather threads by list traversal of active/linked processes 
-        for task in self.filter_tasks(all_tasks):
+        for task in taskmods.DllList(self._config).calculate():
             for thread in task.ThreadListHead.\
                     list_of_type("_ETHREAD", "ThreadListEntry"):
                 seen_threads[thread.obj_vm.vtop(thread.obj_offset)] = (False, thread)
@@ -378,7 +379,7 @@ class Threads(taskmods.DllList):
         for _offset, (found_by_scanner, thread) in seen_threads.items():
 
             # Skip processes the user doesn't want to see
-            if pidlist and thread.Cid.UniqueProcess not in pidlist:
+            if ((self._config.PID or self._config.OFFSET) and not pidlist) or (pidlist and thread.Cid.UniqueProcess not in pidlist):
                 continue
 
             # Do we need to gather DLLs for module resolution 
@@ -417,6 +418,190 @@ class Threads(taskmods.DllList):
             yield thread, addr_space, mods, mod_addrs, \
                         instances, hooked_tables, system_range, owner_name
 
+    def unified_output(self, data):
+        return TreeGrid([("Offset", Address),
+                         ("PID", int),
+                         ("TID", int),
+                         ("Tags", str),
+                         ("Create Time", str),
+                         ("Exit Time", str),
+                         ("Owning Process", str),
+                         ("Attached Process", str),
+                         ("State", str),
+                         ("State Reason", str),
+                         ("Base Priority", int),
+                         ("Priority", int),
+                         ("TEB", Address),
+                         ("Start Address", Address),
+                         ("Owner Name", str),
+                         ("Win32 Start Address", Address),
+                         ("Win32 Thread", Address),
+                         ("Cross Thread Flags", str),
+                         ("EIP", Hex),
+                         ("EAX", Hex),
+                         ("EBX", Hex),
+                         ("ECX", Hex),
+                         ("EDX", Hex),
+                         ("ESI", Hex),
+                         ("EDI", Hex),
+                         ("ESP", Hex),
+                         ("EBP", Hex),
+                         ("ErrCode", Hex),
+                         ("SegCS", Hex),
+                         ("SegSS", Hex),
+                         ("SegDS", Hex),
+                         ("SegES", Hex),
+                         ("SegGS", Hex),
+                         ("SegFS", Hex),
+                         ("EFlags", Hex),
+                         ("dr0", Hex),
+                         ("dr1", Hex),
+                         ("dr2", Hex),
+                         ("dr3", Hex),
+                         ("dr6", Hex),
+                         ("dr7", Hex),
+                         ("SSDT", Address),
+                         ("Entry Number", int),
+                         ("Descriptor Service Table", Address),
+                         ("Hook Number", int),
+                         ("Function Name", str),
+                         ("Function Address", Address),
+                         ("Module Name", str),
+                         ("Disassembly", str),
+                         ],
+                        self.generator(data))
+
+    def generator(self, data):
+        # Determine which filters the user wants to see
+        if self._config.FILTER:
+            filters = set(self._config.FILTER.split(','))
+        else:
+            filters = set()
+
+        for thread, addr_space, mods, mod_addrs, \
+                     instances, hooked_tables, system_range, owner_name in data:
+            # If the user didn't set filters, display all results. If
+            # the user set one or more filters, only show threads
+            # with matching results.
+            tags = set([t for t, v in instances.items() if v.check()])
+
+            if filters and not filters & tags:
+                continue
+
+            values = []
+
+            values.append(Address(thread.obj_offset))
+            values.append(int(thread.Cid.UniqueProcess))
+            values.append(int(thread.Cid.UniqueThread))
+
+            values.append(','.join(tags))
+            values.append(str(thread.CreateTime))
+            if thread.ExitTime > 0:
+                values.append(str(thread.ExitTime))
+            else:
+                values.append('')
+
+            values.append(str(thread.owning_process().ImageFileName))
+
+            values.append(str(thread.attached_process().ImageFileName))
+
+            # Lookup the thread's state
+            state = str(thread.Tcb.State)
+
+            # Find the wait reason
+            if state == 'Waiting':
+                state_reason = str(thread.Tcb.WaitReason)
+            else:
+                state_reason = ''
+
+            values.append(state)
+            values.append(state_reason)
+
+            values.append(int(thread.Tcb.BasePriority))
+            values.append(int(thread.Tcb.Priority))
+            values.append(Address(thread.Tcb.Teb))
+
+            values.append(Address(thread.StartAddress))
+            values.append(owner_name)
+
+            # Check the flag which indicates whether Win32StartAddress is valid
+            if thread.SameThreadApcFlags & 1:
+                values.append(Address(thread.Win32StartAddress))
+            else:
+                values.append(Address(-1))
+
+            values.append(Address(thread.Tcb.Win32Thread))
+            values.append(str(thread.CrossThreadFlags))
+
+            # Disasemble the start address if possible
+            dis = ''
+            process_space = thread.owning_process().get_process_address_space()
+
+            if process_space.is_valid_address(thread.StartAddress):
+                buf = process_space.zread(thread.StartAddress, 24)
+
+                mode = "32bit" if self.bits32 else "64bit"
+
+                dis += "\n".join(["{0:#x} {1:<16} {2}".format(o, h, i)
+                    for o, i, h in malfind.Disassemble(buf, thread.StartAddress.v(), mode)])
+
+            if self.bits32:
+                # Print the registers if possible
+                trapframe = thread.Tcb.TrapFrame.dereference_as("_KTRAP_FRAME")
+
+                if trapframe:
+                    for r in trapframe.Eip, trapframe.Eax, trapframe.Ebx, \
+                            trapframe.Ecx, trapframe.Edx, trapframe.Esi, \
+                            trapframe.Edi, trapframe.HardwareEsp, \
+                            trapframe.Ebp, trapframe.ErrCode, trapframe.SegCs, \
+                            trapframe.HardwareSegSs, trapframe.SegDs, \
+                            trapframe.SegEs, trapframe.SegGs, trapframe.SegFs, \
+                            trapframe.EFlags, trapframe.Dr0, trapframe.Dr1, \
+                            trapframe.Dr2, trapframe.Dr3, trapframe.Dr6, \
+                            trapframe.Dr7 :
+                        values.append(Hex(r))
+                else:
+                    values.extend( [Hex(-1)] * 23 )
+
+                values.append(Address(thread.Tcb.ServiceTable))
+
+                ssdt_obj = obj.Object("_SERVICE_DESCRIPTOR_TABLE",
+                    offset = thread.Tcb.ServiceTable,
+                    vm = addr_space
+                    )
+
+                if ssdt_obj != None:
+                    for i, desc in enumerate(ssdt_obj.Descriptors):
+                        if desc.is_valid():
+                            service_table = Address(desc.KiServiceTable.v())
+                        else:
+                            service_table = Address(-1)
+                        # Show exactly which functions are hooked
+                        table = desc.KiServiceTable.v()
+                        if table not in hooked_tables.keys():
+                            yield (0, values + [i, service_table, -1, '',
+                                                Address(-1), '', dis])
+                            continue
+                        yielded=False
+                        for (j, func_name, func_addr, mod_name) in hooked_tables[table]:
+                            yielded=True
+                            yield(0, values + [i, service_table, j, func_name,
+                                               Address(func_addr), mod_name, dis])
+                        if not yielded:
+                            yield (0, values + [i, service_table, -1, '',
+                                                Address(-1), '', dis])
+                else:
+                    values.extend([ -1, Address(-1), -1, '', Address(-1), '', dis ])
+                    yield (0, values)
+            else:
+                # registers
+                values.extend( [Hex(-1)] * 23 )
+                # ssdt
+                values.extend([ Address(-1), -1, Address(-1), -1, '',
+                                Address(-1), '', dis ])
+                yield (0, values)
+
+
     def render_text(self, outfd, data):
 
         # Determine which filters the user wants to see
@@ -489,9 +674,9 @@ class Threads(taskmods.DllList):
                         table = desc.KiServiceTable.v()
                         if table not in hooked_tables.keys():
                             continue
-                        for (i, func_name, func_addr, mod_name) in hooked_tables[table]:
+                        for (j, func_name, func_addr, mod_name) in hooked_tables[table]:
                             s += "      [{0:#x}] {1} {2:#x} {3}\n".format(
-                                i, func_name, func_addr, mod_name)
+                                j, func_name, func_addr, mod_name)
 
             s += "Win32Thread: {0:#010x}\n".format(thread.Tcb.Win32Thread)
             s += "CrossThreadFlags: {0}\n".format(thread.CrossThreadFlags)
