@@ -37,6 +37,7 @@ import volatility.exceptions as exceptions
 import volatility.obj as obj
 import volatility.debug as debug
 import volatility.dwarf as dwarf
+import volatility.scan as scan
 import volatility.plugins.linux.common as linux_common
 import volatility.plugins.linux.flags as linux_flags
 import volatility.addrspace as addrspace
@@ -211,11 +212,18 @@ def LinuxProfileFactory(profpkg):
             # change the name to catch any code referencing the old hash table
             self.sys_map = {}
             self.sym_addr_cache = {}
+            self.dentry_cache = {}
+            self.physical_shift = 0
+            self.virtual_shift = 0
             obj.Profile.__init__(self, *args, **kwargs)
 
         def clear(self):
             """Clear out the system map, and everything else"""
             self.sys_map = {}
+            self.sym_addr_cache = {}
+            self.dentry_cache = {}
+            self.physical_shift = 0
+            self.virtual_shift = 0
             obj.Profile.clear(self)
 
         def reset(self):
@@ -281,7 +289,11 @@ def LinuxProfileFactory(profpkg):
                 mod = symtable[module]
 
                 for (name, addrs) in mod.items():
-                    ret.append(addrs)
+                    addr = addrs[0][0]
+                    if self.virtual_shift and addr:
+                        addr = addr + self.virtual_shift
+
+                    ret.append((name, addr))
             else:
                 debug.info("All symbols requested for non-existent module %s" % module)
 
@@ -292,14 +304,12 @@ def LinuxProfileFactory(profpkg):
 
             # returns a hash table for quick looks
             # the main use of this function is to see if an address is known
+            symbols = self.get_all_symbols(module)
+            
             ret = {}
 
-            symbols = self.get_all_symbols(module)
-
-            for sym in symbols:
-
-                for (addr, addrtype) in sym:
-                    ret[addr] = 1
+            for _name, addr in symbols:
+                ret[addr] = 1
 
             return ret
 
@@ -312,7 +322,7 @@ def LinuxProfileFactory(profpkg):
             for (name, addrs) in mod.items():
 
                 for (addr, addr_type) in addrs:
-                    if sym_address == addr:
+                    if sym_address == addr + self.virtual_shift:
                         ret = name
                         break
 
@@ -413,6 +423,9 @@ def LinuxProfileFactory(profpkg):
             else:
                 debug.info("Requested module {0:s} not found in symbol table\n".format(module))
 
+            if ret:
+                ret = ret + self.virtual_shift
+
             return ret
 
         def get_symbol_type(self, sym_name, nm_type = "", module = "kernel"):
@@ -710,7 +723,13 @@ class inet_sock(obj.CType):
     @property
     def state(self):
         state = self.sk.__sk_common.skc_state #pylint: disable-msg=W0212
-        return linux_flags.tcp_states[state]
+
+        if 0 <= state < len(linux_flags.tcp_states):
+            ret = linux_flags.tcp_states[state]
+        else:
+            ret = "" 
+
+        return ret
     
     @property
     def src_port(self):
@@ -723,12 +742,14 @@ class inet_sock(obj.CType):
 
     @property
     def dst_port(self):
-        if hasattr(self, "dport"):
+        if hasattr(self, "sk") and hasattr(self.sk, "__sk_common") and hasattr(self.sk.__sk_common, "skc_portpair"):
+            return socket.htons(self.sk.__sk_common.skc_portpair & 0xffff) #pylint: disable-msg=W0212  
+        elif hasattr(self, "dport"):
             return socket.htons(self.dport)
         elif hasattr(self, "inet_dport"):
             return socket.htons(self.inet_dport)
         elif hasattr(self, "sk") and hasattr(self.sk, "__sk_common") and hasattr(self.sk.__sk_common, "skc_dport"):
-            return self.sk.__sk_common.skc_dport
+            return socket.htons(self.sk.__sk_common.skc_dport) #pylint: disable-msg=W0212
         else:
             return None
 
@@ -812,6 +833,61 @@ class net_device(obj.CType):
         return self.flags & 0x100 == 0x100 # IFF_PROMISC
 
 class module_struct(obj.CType):
+    @property   
+    def module_core(self):
+        if hasattr(self, "core_layout"):
+            ret = self.m("core_layout").m("base")
+        else:
+            ret = self.m("module_core")
+
+        return ret
+
+    @property
+    def module_init(self):
+        if hasattr(self, "init_layout"):
+            ret = self.m("init_layout").m("base")
+        else:
+            ret = self.m("module_init")
+    
+        return ret
+
+    @property
+    def init_size(self):
+        if hasattr(self, "init_layout"):
+            ret = self.m("init_layout").m("size")
+        else:
+            ret = self.m("init_size")
+
+        return ret
+ 
+    @property
+    def init_text_size(self):
+        if hasattr(self, "init_layout"):
+            ret = self.m("init_layout").m("text_size")
+        else:
+            ret = self.m("init_text_size")
+
+        return ret
+ 
+    @property 
+    def core_text_size(self):
+        if hasattr(self, "core_layout"):
+            ret = self.m("core_layout").m("text_size")
+        else:
+            ret = self.m("core_text_size")
+
+        return ret
+    
+    @property 
+    def core_size(self):
+        if hasattr(self, "core_layout"):
+            ret = self.m("core_layout").m("size")
+        else:
+            ret = self.m("core_size")
+
+        return ret
+   
+
     def _get_sect_count(self, grp):
         arr = obj.Object(theType = 'Array', offset = grp.attrs, vm = self.obj_vm, targetType = 'Pointer', count = 25)
 
@@ -871,14 +947,14 @@ class module_struct(obj.CType):
                 val = val + str(mret or '')
 
         elif getfn == self.obj_vm.profile.get_symbol("param_get_string"):
-            val = param.str.dereference_as("String", length = param.str.maxlen)
+            val = str(param.str.dereference_as("String", length = param.str.maxlen))
 
         elif getfn == self.obj_vm.profile.get_symbol("param_get_charp"):
             addr = obj.Object("Pointer", offset = param.arg, vm = self.obj_vm)
             if addr == 0:
                 val = "(null)"
             else:
-                val = addr.dereference_as("String", length = 256)
+                val = str(addr.dereference_as("String", length = 256))
 
         elif getfn.v() in ints:
             val = obj.Object(ints[getfn.v()], offset = param.arg, vm = self.obj_vm)
@@ -889,14 +965,15 @@ class module_struct(obj.CType):
                 else:
                     val = 'N'
 
-            if getfn == self.obj_vm.profile.get_symbol("param_get_invbool"):
+            elif getfn == self.obj_vm.profile.get_symbol("param_get_invbool"):
                 if val:
                     val = 'N'
                 else:
                     val = 'Y'
+            else:
+                val = int(val)
 
         else:
-            print "Unknown get_fn: {0:#x}".format(getfn)
             return None
 
         return val
@@ -959,6 +1036,24 @@ class module_struct(obj.CType):
                 break
 
         return ret       
+   
+    @property
+    def symtab(self):
+        if hasattr(self, "kallsyms"):
+            ret = self.kallsyms.symtab
+        else:
+            ret = self.m("symtab")
+
+        return ret 
+ 
+    @property
+    def num_symtab(self):
+        if hasattr(self, "kallsyms"):
+            ret = self.kallsyms.num_symtab.v()
+        else:
+            ret = self.m("num_symtab").v()
+
+        return ret   
 
     def is_valid(self):
         valid = False
@@ -985,9 +1080,29 @@ class module_struct(obj.CType):
         return valid
 
 class vm_area_struct(obj.CType):
+    def is_valid(self):
+        start = self.vm_start.v()
+        end   = self.vm_end.v()
+        pgoff = self.vm_pgoff.v()
+
+        valid = True
+
+        if  (start > end) or \
+            (end - start > 100000000000) or \
+            (start > 0xff00000000000000) or \
+            (end > 0xff00000000000000) or \
+            (pgoff > 100000000000):
+
+            valid = False
+
+        return valid           
+
     def vm_name(self, task):
         if self.vm_file:
             fname = linux_common.get_path(task, self.vm_file)
+            if fname == []:
+                fname = ""
+
         elif self.vm_start <= task.mm.start_brk and self.vm_end >= task.mm.brk:
             fname = "[heap]"
         elif self.vm_start <= task.mm.start_stack and self.vm_end >= task.mm.start_stack:
@@ -1057,8 +1172,8 @@ class vm_area_struct(obj.CType):
         ret = False        
 
         flags_str  = self.protection()
-       
-        if flags_str == "VM_READ|VM_WRITE|VM_EXEC":
+      
+        if flags_str.find("VM_READ|VM_WRITE|VM_EXEC") != -1:
             ret = True 
             
         elif flags_str == "VM_READ|VM_EXEC" and not self.vm_file:
@@ -1068,10 +1183,14 @@ class vm_area_struct(obj.CType):
 
     def info(self, task):
         if self.vm_file:
-            inode = self.vm_file.dentry.d_inode
-            major, minor = inode.i_sb.major, inode.i_sb.minor
-            ino = inode.i_ino
             pgoff = self.vm_pgoff << 12
+            
+            inode = self.vm_file.dentry.d_inode
+            if inode and inode.is_valid():
+                major, minor = inode.i_sb.major, inode.i_sb.minor
+                ino = inode.i_ino
+            else:
+                major, minor, ino = [0] * 3
         else:
             (major, minor, ino, pgoff) = [0] * 4
 
@@ -1082,6 +1201,16 @@ class vm_area_struct(obj.CType):
 
         return fname, major, minor, ino, pgoff 
 
+class kobject(obj.CType):
+    def reference_count(self):
+        refcnt = self.kref.refcount
+        if hasattr(refcnt, "counter"):
+            ret = refcnt.counter
+        else:
+            ret = refcnt.refs.counter
+
+        return ret
+
 class task_struct(obj.CType):
     def is_valid_task(self):
 
@@ -1091,6 +1220,11 @@ class task_struct(obj.CType):
             ret = self.cred.is_valid()
 
         return ret
+    
+    @property
+    def comm(self):
+        c = self.m("comm")
+        return c.replace("\x1b", "\\x1b")
 
     def getcwd(self):
         rdentry = self.fs.get_root_dentry()
@@ -1111,6 +1245,9 @@ class task_struct(obj.CType):
 
         proc_as = self.get_process_address_space()
 
+        if proc_as == None:
+            return ret
+  
         elf_hdr = obj.Object("elf_hdr", offset = elf_addr, vm = proc_as)
 
         if not elf_hdr.is_valid():
@@ -1132,6 +1269,9 @@ class task_struct(obj.CType):
 
             real_size = end - start
 
+            if real_size < 0 or real_size > 100000000:
+                continue
+
             sects[start] = real_size
  
         last_end = -1
@@ -1159,6 +1299,12 @@ class task_struct(obj.CType):
         else:
             ret = self.m("uid")
 
+        if type(ret) in [obj.CType, obj.NativeType]:
+            ret = ret.v()
+
+        if ret > 1000000:
+            ret = -1
+
         return ret
 
     @property
@@ -1175,6 +1321,12 @@ class task_struct(obj.CType):
         else:
             ret = self.m("gid")
 
+        if type(ret) == obj.CType:
+            ret = ret.v()
+
+        if ret > 1000000:
+            ret = -1
+
         return ret
 
     @property
@@ -1185,6 +1337,9 @@ class task_struct(obj.CType):
         else:
             ret = self.m("euid")
 
+        if type(ret) == obj.CType:
+            ret = ret.v()
+
         return ret
 
     def find_heap_vma(self):
@@ -1208,6 +1363,8 @@ class task_struct(obj.CType):
             return
 
         proc_as = self.get_process_address_space()
+        if proc_as == None:
+            return
 
         for off in self.search_process_memory(["\x40\x00\x00\x00"], heap_only=True):
             # test the number of buckets
@@ -1224,8 +1381,7 @@ class task_struct(obj.CType):
         seen_starts = {}
 
         proc_as = self.get_process_address_space()        
-
-        if not proc_as:
+        if proc_as == None:
             return
 
         # get libraries from proc_maps
@@ -1458,7 +1614,6 @@ class task_struct(obj.CType):
 
     def bash_history_entries(self):
         proc_as = self.get_process_address_space()
-
         if not proc_as:
             return
 
@@ -1494,48 +1649,36 @@ class task_struct(obj.CType):
         for hist in sorted(history_entries, key = attrgetter('time_as_integer')):
             yield hist              
 
-    def psenv(self):
-        env = ""
-
-        if self.mm:
-            # set the as with our new dtb so we can read from userland
-            proc_as = self.get_process_address_space()
-
-            # read argv from userland
-            start = self.mm.env_start.v()
-
-            env = proc_as.read(start, self.mm.env_end - self.mm.env_start + 10)
-            
-        if env:
-            ents = env.split("\x00")
-            for varstr in ents:
-                eqidx = varstr.find("=")
-
-                if eqidx == -1:
-                    continue
-
-                key = varstr[:eqidx]
-                val = varstr[eqidx+1:]
-
-                yield (key, val) 
-
     def _dynamic_env(self, proc_as, pack_format, addr_sz):
+        # preload address 0
+        addr_cache = {0 : 1}
+
         for vma in self.get_proc_maps():
             if not (vma.vm_file and str(vma.vm_flags) == "rw-"):
                 continue
             
             fname = vma.info(self)[0]
 
-            if fname.find("ld") == -1 and fname != "/bin/bash":
+            if fname.find("ld") == -1 and (not fname.endswith(("/bin/bash", "/bin/dash", "/bin/sh"))):
                 continue
 
             env_start = 0
-            for off in range(vma.vm_start, vma.vm_end):
+       
+            vma_start = int(vma.vm_start)
+            vma_end = int(vma.vm_end)
+            vma_len = vma_end - vma_start
+            vma_data = proc_as.zread(vma_start, vma_len)
+
+            for off in range(0, vma_len - addr_sz, 4):
                 # check the first index
-                addrstr = proc_as.read(off, addr_sz)
-                if not addrstr or len(addrstr) != addr_sz:
-                    continue
+                addrstr = vma_data[off:off+addr_sz]
                 addr = struct.unpack(pack_format, addrstr)[0]
+
+                if addr in addr_cache:
+                    continue
+
+                addr_cache[addr] = 1
+
                 # check first idx...
                 if addr:
                     firstaddrstr = proc_as.read(addr, addr_sz)
@@ -1586,12 +1729,17 @@ class task_struct(obj.CType):
                         break
 
     def _shell_variables(self, proc_as, pack_format, addr_sz):
+        # preload cache with address 0
+        ptr_cache = {0 : 1}
+            
+        nbuckets_offset = self.obj_vm.profile.get_obj_offset("_bash_hash_table", "nbuckets") 
+
         bash_was_last = False
         for vma in self.get_proc_maps():
             if vma.vm_file:
                 fname = vma.info(self)[0]
        
-                if fname.endswith("/bin/bash"):
+                if fname.endswith(("/bin/bash", "/bin/dash", "/bin/sh")):
                     bash_was_last = True
                 else:
                     bash_was_last = False
@@ -1604,16 +1752,21 @@ class task_struct(obj.CType):
             if bash_was_last == False:
                 continue
         
-            nbuckets_offset = self.obj_vm.profile.get_obj_offset("_bash_hash_table", "nbuckets") 
+            vma_start = int(vma.vm_start)
+            vma_end = int(vma.vm_end)
+            vma_len = vma_end - vma_start
+            vma_data = proc_as.zread(vma_start, vma_len)
 
-            for off in range(vma.vm_start, vma.vm_end, 4):
-                ptr_test = proc_as.read(off, 4)
-                if not ptr_test:
+            for off in range(0, vma_len - addr_sz, 4):
+                ptr_test = vma_data[off:off+addr_sz]
+                
+                ptr = struct.unpack(pack_format, ptr_test)[0]
+                if ptr in ptr_cache:
                     continue
 
-                ptr = struct.unpack(pack_format, ptr_test)[0]
-                
-                ptr_test2 = proc_as.read(ptr + 20, 4)
+                ptr_cache[ptr] = 1
+
+                ptr_test2 = proc_as.read(ptr + 20, addr_sz)
                 if not ptr_test2:
                     continue
 
@@ -1635,10 +1788,12 @@ class task_struct(obj.CType):
 
     def bash_environment(self):
         proc_as = self.get_process_address_space()
-        
         # In cases when mm is an invalid pointer 
         if not proc_as:
             return
+        
+        if str(self.comm) not in ["sh", "dash", "bash"]:
+            return
 
         # Are we dealing with 32 or 64-bit pointers
         if self.obj_vm.profile.metadata.get('memory_model', '32bit') == '32bit':
@@ -1691,7 +1846,7 @@ class task_struct(obj.CType):
                     state = inet_sock.state if inet_sock.protocol == "TCP" else ""
                     family = inet_sock.sk.__sk_common.skc_family #pylint: disable-msg=W0212
 
-                    if family == socket.AF_UNIX:
+                    if family == 1: # AF_UNIX
                         unix_sock = obj.Object("unix_sock", offset = inet_sock.sk.v(), vm = self.obj_vm)
 
                         if unix_sock.addr:
@@ -1700,9 +1855,9 @@ class task_struct(obj.CType):
                         else:
                             name = ""
 
-                        yield (socket.AF_UNIX, (name, iaddr.i_ino))
+                        yield (1, (name, iaddr.i_ino))
 
-                    elif family in (socket.AF_INET, socket.AF_INET6):
+                    elif family in (socket.AF_INET, socket.AF_INET6, 10, 30):
                         sport = inet_sock.src_port 
                         dport = inet_sock.dst_port 
                         saddr = inet_sock.src_addr
@@ -1733,15 +1888,16 @@ class task_struct(obj.CType):
 
     def get_libdl_maps(self):
         proc_as = self.get_process_address_space()
-        
+        if proc_as == None:
+            return       
+ 
         found_list = False
 
         for vma in self.get_proc_maps():
             # find the executable part of libdl
             ehdr = obj.Object("elf_hdr", offset = vma.vm_start, vm = proc_as)
 
-            if not ehdr.is_valid():
-                #print "could not get header for  %d | %s" % (self.pid, self.comm)
+            if not ehdr or not ehdr.is_valid():
                 continue
 
             for phdr in ehdr.program_headers():
@@ -1770,16 +1926,26 @@ class task_struct(obj.CType):
         thread_offset = self.obj_vm.profile.get_obj_offset("task_struct", "thread_group")
         threads = [self]
         x = obj.Object('task_struct', self.thread_group.next.v() - thread_offset, self.obj_vm)
-        while x not in threads and x.thread_group.next.is_valid():
+        while x not in threads and x.is_valid() and x.thread_group.is_valid() and x.thread_group.next.is_valid():
             threads.append(x)
             x = obj.Object('task_struct', x.thread_group.next.v() - thread_offset, self.obj_vm)
         return threads
 
     def get_proc_maps(self):
-        if not self.mm:
+        if not self.mm or self.get_process_address_space() == None:
             return
+        seen = {}
         for vma in linux_common.walk_internal_list("vm_area_struct", "vm_next", self.mm.mmap):
+            val = vma.v()
+            if val in seen:
+                break
+           
+            if not vma.is_valid():
+                break
+ 
             yield vma
+
+            seen[val] = 1
    
     def _walk_rb(self, rb):
         if not rb.is_valid():
@@ -1823,6 +1989,8 @@ class task_struct(obj.CType):
         scan_blk_sz = 1024 * 1024 * 10
 
         addr_space = self.get_process_address_space()
+        if addr_space == None:
+            return
 
         for vma in self.get_proc_maps():
             if heap_only:
@@ -1844,6 +2012,8 @@ class task_struct(obj.CType):
 
     def elfs(self):
         proc_as = self.get_process_address_space()
+        if proc_as == None:
+            return
 
         for vma in self.get_proc_maps():
             elf = obj.Object("elf_hdr", offset = vma.vm_start, vm = proc_as) 
@@ -1926,59 +2096,13 @@ class task_struct(obj.CType):
 
         return self.SH_DIV(1000000 * 1000, self.ACTHZ(CLOCK_TICK_RATE, HZ), 8)
 
-    def get_time_vars(self):
-        '''
-        Sometime in 3.[3-5], Linux switched to a global timekeeper structure
-        This just figures out which is in use and returns the correct variables
-        '''
-
-        wall_addr       = self.obj_vm.profile.get_symbol("wall_to_monotonic")
-        sleep_addr      = self.obj_vm.profile.get_symbol("total_sleep_time")
-        timekeeper_addr = self.obj_vm.profile.get_symbol("timekeeper")
-        tkcore_addr     = self.obj_vm.profile.get_symbol("tk_core") 
-
-        # old way
-        if wall_addr and sleep_addr:
-            wall = obj.Object("timespec", offset = wall_addr, vm = self.obj_vm)
-            timeo = obj.Object("timespec", offset = sleep_addr, vm = self.obj_vm)
-
-        elif wall_addr:
-            wall  = obj.Object("timespec", offset = wall_addr, vm = self.obj_vm)
-            timeo = linux_common.vol_timespec(0, 0)
-    
-        # timekeeper way
-        elif timekeeper_addr:
-            timekeeper = obj.Object("timekeeper", offset = timekeeper_addr, vm = self.obj_vm)
-            wall = timekeeper.wall_to_monotonic
-            timeo = timekeeper.total_sleep_time
-
-        # 3.17(ish) - 3.19(ish) way
-        elif tkcore_addr and hasattr("timekeeper", "total_sleep_time"):
-            # skip seqcount
-            timekeeper = obj.Object("timekeeper", offset = tkcore_addr + 4, vm = self.obj_vm)
-            wall = timekeeper.wall_to_monotonic
-            timeo = timekeeper.total_sleep_time
-
-        # 3.19(ish)+
-        # getboottime from 3.19.x
-        elif tkcore_addr:
-            # skip seqcount
-            timekeeper = obj.Object("timekeeper", offset = tkcore_addr + 8, vm = self.obj_vm)
-            wall = timekeeper.wall_to_monotonic
-
-            oreal = timekeeper.offs_real
-            oboot = timekeeper.offs_boot
- 
-            tv64 = (oreal.tv64 & 0xffffffff) - (oboot.tv64 & 0xffffffff)
-
-            tv64 = (tv64 / 100000000) * -1
-            timeo = linux_common.vol_timespec(tv64, 0) 
-
-        return (wall, timeo)
-
     # based on 2.6.35 getboottime
     def get_boot_time(self):
-        (wall, timeo) = self.get_time_vars()
+        (wall, timeo) = linux_common.get_time_vars(self.obj_vm)
+
+        if wall == None or timeo == None:
+            return -1
+
         secs = wall.tv_sec + timeo.tv_sec
         nsecs = wall.tv_nsec + timeo.tv_nsec
 
@@ -1998,7 +2122,6 @@ class task_struct(obj.CType):
         return boot_time
         
     def get_task_start_time(self):
-
         if hasattr(self, "real_start_time"):
             start_time = self.real_start_time
         else:
@@ -2009,39 +2132,50 @@ class task_struct(obj.CType):
 
         start_secs = start_time.tv_sec + (start_time.tv_nsec / linux_common.nsecs_per / 100)
 
-        sec = self.get_boot_time() + start_secs
-               
-        # convert the integer as little endian 
-        try:
-            data = struct.pack("<I", sec)
-        except struct.error, e:
-            # in case we exceed 0 <= number <= 4294967295
-            return 0
+        boot_time =  self.get_boot_time()
+       
+        if boot_time != -1:
+            sec = boot_time + start_secs
 
-        bufferas = addrspace.BufferAddressSpace(self.obj_vm.get_config(), data = data)
-        dt = obj.Object("UnixTimeStamp", offset = 0, vm = bufferas, is_utc = True)
+            # convert the integer as little endian 
+            try:
+                data = struct.pack("<I", sec)
+            except struct.error, e:
+                # in case we exceed 0 <= number <= 4294967295
+                return 0
 
+            bufferas = addrspace.BufferAddressSpace(self.obj_vm.get_config(), data = data)
+            dt = obj.Object("UnixTimeStamp", offset = 0, vm = bufferas, is_utc = True)
+        else:
+            dt = None
+        
         return dt
 
-    def get_environment(self):
+    def psenv(self): 
         if self.mm:
             # set the as with our new dtb so we can read from userland
             proc_as = self.get_process_address_space()
+            if proc_as == None:
+                return
 
-            # read argv from userland
             start = self.mm.env_start.v()
 
-            argv = proc_as.read(start, self.mm.env_end - self.mm.env_start + 10)
-            
-            if argv:
-                # split the \x00 buffer into args
-                env = " ".join(argv.split("\x00"))
+            size_to_read = self.mm.env_end.v() - start + 10 
 
-            else:
-                env = ""
-        else:
-            # kernel thread
-            env = ""
+            if 4 < size_to_read < 4096:
+                args = proc_as.read(start, size_to_read)
+                if args:
+                    for vals in args.split("\x00"): 
+                        ents = vals.split("=")
+
+                        if len(ents) == 2:
+                            yield ents[0], ents[1]        
+
+    def get_environment(self):
+        env = ""
+
+        for key, value in self.psenv():
+            env = env + "%s=%s " % (key, value)
 
         if len(env) > 1 and env[-1] == " ":
             env = env[:-1]
@@ -2052,17 +2186,24 @@ class task_struct(obj.CType):
         if self.mm:
             # set the as with our new dtb so we can read from userland
             proc_as = self.get_process_address_space()
+            if proc_as == None:
+                return ""
 
             # read argv from userland
             start = self.mm.arg_start.v()
 
-            argv = proc_as.read(start, self.mm.arg_end - self.mm.arg_start)
+            size_to_read = self.mm.arg_end - self.mm.arg_start
 
-            if argv:
-                # split the \x00 buffer into args
-                name = " ".join(argv.split("\x00"))
-            else:
+            if size_to_read < 1 or size_to_read > 4096:
                 name = ""
+            else:
+                argv = proc_as.read(start, size_to_read)
+
+                if argv:
+                    # split the \x00 buffer into args
+                    name = " ".join(argv.split("\x00"))
+                else:
+                    name = ""
         else:
             # kernel thread
             name = "[" + self.comm + "]"
@@ -2167,39 +2308,132 @@ class dentry(obj.CType):
             ret = self.m("d_count")
         return ret
 
+class swapperScan(scan.BaseScanner):
+    """ Scanner for swapper string for Mountain Lion """
+    checks = []
+
+    def __init__(self, needles = None):
+        self.needles = needles
+        self.checks = [ ("MultiStringFinderCheck", {'needles':needles}) ]
+        scan.BaseScanner.__init__(self) 
+
+    def scan(self, address_space, offset = 0, maxlen = None):
+        for offset in scan.BaseScanner.scan(self, address_space, offset, maxlen):
+            yield offset
+
 class VolatilityDTB(obj.VolatilityMagic):
     """A scanner for DTB values."""
 
     def generate_suggestions(self):
         """Tries to locate the DTB."""
         profile = self.obj_vm.profile
+        config = self.obj_vm.get_config()
+        tbl    = self.obj_vm.profile.sys_map["kernel"]
         
         if profile.metadata.get('memory_model', '32bit') == "32bit":
-            sym   = "swapper_pg_dir"
-            shifts = [0xc0000000]
+            sym     = "swapper_pg_dir"
+            shifts  = [0xc0000000]
+            read_sz = 4
+            fmt     = "<I"
         else:
-            sym   = "init_level4_pgt"
-            shifts = [0xffffffff80000000, 0xffffffff80000000 - 0x1000000, 0xffffffff7fe00000]       
+            sym     = "init_level4_pgt"
+            # >= 4.13 
+            if not sym in tbl:
+                sym = "init_top_pgt"
+                
+            shifts  = [0xffffffff80000000, 0xffffffff80000000 - 0x1000000, 0xffffffff7fe00000]       
+            read_sz = 8
+            fmt     = "<Q"
+       
+        if config.PHYSICAL_SHIFT and not config.VIRTUAL_SHIFT:
+            debug.error("You must specifiy both the virtual and physical shift.") 
+        elif not config.PHYSICAL_SHIFT and config.VIRTUAL_SHIFT:
+            debug.error("You must specifiy both the virtual and physical shift.") 
+        elif config.PHYSICAL_SHIFT and config.VIRTUAL_SHIFT:
+            physical_shift_address = config.PHYSICAL_SHIFT
+            virtual_shift_address = config.VIRTUAL_SHIFT
+        else:
+            physical_shift_address = self.obj_vm.profile.physical_shift
+            virtual_shift_address  = self.obj_vm.profile.virtual_shift    
 
         good_dtb = -1
- 
-        for shift in shifts:
-            sym_addr = profile.get_symbol(sym) - shift
-
-            pas = self.obj_vm
-
-            init_task_addr = profile.get_symbol("init_task") 
-            offset         = profile.get_obj_offset("task_struct", "comm")
+            
+        init_task_addr = tbl["init_task"][0][0] + virtual_shift_address
+        dtb_sym_addr   = tbl[sym][0][0] + virtual_shift_address
+        files_sym_addr = tbl["init_files"][0][0] + virtual_shift_address
        
-            read_addr = init_task_addr - shift + offset
+        comm_offset   = profile.get_obj_offset("task_struct", "comm")
+        pid_offset    = profile.get_obj_offset("task_struct", "pid")
+        state_offset  = profile.get_obj_offset("task_struct", "state")
+        files_offset  = profile.get_obj_offset("task_struct", "files") 
+        mm_offset     = profile.get_obj_offset("task_struct", "active_mm")
+        
+        # this appeared around 2.6.24, which we only need it for samples with KASLR, which came much later
+        try:
+            sched_class_offset = profile.get_obj_offset("task_struct", "sched_class")
+            idle_class_addr    = tbl["idle_sched_class"][0][0] + virtual_shift_address
+        except KeyError:
+            sched_class_offset = -1
+            idle_class_addr    = -1
 
-            buf = pas.read(read_addr, 12)        
-          
-            if buf:
-                idx = buf.find("swapper")
-                if idx == 0:
-                    good_dtb = sym_addr
-                    break
+        pas           = self.obj_vm
+
+        if physical_shift_address != 0 and virtual_shift_address != 0:
+            good_dtb = (dtb_sym_addr - shifts[0] - virtual_shift_address) + physical_shift_address
+            self.obj_vm.profile.physical_shift = physical_shift_address 
+            self.obj_vm.profile.virtual_shift  = virtual_shift_address
+
+        if good_dtb == -1:
+            for shift in shifts:
+                sym_addr = dtb_sym_addr - shift
+           
+                read_addr = init_task_addr - shift + comm_offset
+
+                buf = pas.read(read_addr, 12)        
+                if buf:
+                    idx = buf.find("swapper")
+                    if idx == 0:
+                        good_dtb = sym_addr
+                        break
+
+        # check for relocated or physical aslr kernel
+        if good_dtb == -1:
+            scanner = swapperScan(needles = ["swapper/0\x00\x00\x00\x00\x00\x00"])
+            ctr = 0
+            for swapper_offset in scanner.scan(self.obj_vm):
+                swapper_address = swapper_offset - comm_offset
+
+                if pas.read(swapper_address + state_offset, 4) != "\x00\x00\x00\x00":
+                    continue
+
+                if pas.read(swapper_address + pid_offset, 4) != "\x00\x00\x00\x00":
+                    continue
+
+                tmp_physical_shift = swapper_address - (init_task_addr - shifts[0]) 
+                if tmp_physical_shift & 0xfff != 0x000:
+                    continue
+
+                good_dtb = (dtb_sym_addr - shifts[0] + 0) + tmp_physical_shift 
+
+                if pas.zread(good_dtb, 8) != "\x00\x00\x00\x00\x00\x00\x00\x00":
+                    continue
+
+                if sched_class_offset != -1:
+                    sched_class_val = pas.read(swapper_address + sched_class_offset, read_sz)
+                    sched_class_addr = struct.unpack(fmt, sched_class_val)[0]
+ 
+                    if (sched_class_addr & 0xfff) != (idle_class_addr & 0xfff):
+                        continue
+
+                files_buf  = pas.read(swapper_address + files_offset, read_sz)
+                files_addr = struct.unpack(fmt, files_buf)[0]
+
+                tmp_virtual_shift = files_addr - files_sym_addr
+
+                self.obj_vm.profile.physical_shift = tmp_physical_shift
+                self.obj_vm.profile.virtual_shift  = tmp_virtual_shift
+ 
+                break
 
         yield good_dtb
 
@@ -2210,17 +2444,19 @@ class VolatilityLinuxIntelValidAS(obj.VolatilityMagic):
     def generate_suggestions(self):
 
         init_task_addr = self.obj_vm.profile.get_symbol("init_task")
-
         if self.obj_vm.profile.metadata.get('memory_model', '32bit') == "32bit":
             shifts = [0xc0000000]
         else:
             shifts = [0xffffffff80000000, 0xffffffff80000000 - 0x1000000, 0xffffffff7fe00000]       
 
         ret = False
+           
+        phys  = self.obj_vm.vtop(init_task_addr)
+        if phys == None:
+            return
 
         for shift in shifts:
-            phys  = self.obj_vm.vtop(init_task_addr)
-            check = init_task_addr - shift
+            check = init_task_addr - shift + self.obj_vm.profile.physical_shift - self.obj_vm.profile.virtual_shift
             if phys == check:
                 ret = True
                 break
@@ -2235,6 +2471,9 @@ class VolatilityLinuxARMValidAS(obj.VolatilityMagic):
         init_task_addr = self.obj_vm.profile.get_symbol("init_task")
         do_fork_addr   = self.obj_vm.profile.get_symbol("do_fork") 
 
+        if not do_fork_addr or not init_task_addr:
+            return
+
         sym_addr_diff = (do_fork_addr - init_task_addr)
 
         if self.obj_vm.profile.metadata.get('memory_model', '32bit') == "32bit":
@@ -2266,6 +2505,7 @@ class LinuxObjectClasses(obj.ProfileModification):
             'hlist_node': hlist_node,
             'files_struct': files_struct,
             'task_struct': task_struct,
+            'kobject'    : kobject,
             'vm_area_struct': vm_area_struct,
             'module' : module_struct,
             'hlist_bl_node' : hlist_bl_node,
@@ -2321,14 +2561,18 @@ class page(obj.CType):
             mem_map_ptr = obj.Object("Pointer", offset = mem_map_addr, vm = self.obj_vm, parent = self.obj_parent)
 
         elif mem_section_addr:
-            # this is hardcoded in the kernel - VMEMMAPSTART, usually 64 bit kernels
-            mem_map_ptr = 0xffffea0000000000
-
+            mem_map_ptr_addr = self.obj_vm.profile.get_symbol("vmemmap_base")
+            if mem_map_ptr_addr:
+                # (later) ASLR kernels
+                mem_map_ptr = obj.Object("unsigned long", offset = mem_map_ptr_addr, vm = self.obj_vm)
+            else:
+                # this is hardcoded in the kernel - VMEMMAPSTART, usually 64 bit kernels
+                mem_map_ptr = 0xffffea0000000000
         else:
             debug.error("phys_addr_of_page: Unable to determine physical address of page. NUMA is not supported at this time.\n")
-
+        
         phys_offset = (self.obj_offset - mem_map_ptr) / self.obj_vm.profile.get_obj_size("page")
-
+        
         phys_offset = phys_offset << 12
 
         return phys_offset
