@@ -3,10 +3,9 @@
 # This file is part of Volatility.
 #
 # Volatility is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License Version 2 as
-# published by the Free Software Foundation.  You may not use, modify or
-# distribute this program under any other version of the GNU General
-# Public License.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # Volatility is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -242,6 +241,62 @@ tcpip_vtypes_vista_64 = {
     }],
 }
 
+# Structures for netscan on x64 Windows 10
+tcpip_vtypes_win_10_x64 = {
+    '_IN_ADDR' : [ None, {
+    'addr4' : [ 0x0, ['IpAddress']],
+    'addr6' : [ 0x0, ['Ipv6Address']],
+    }],
+    '_INETAF' : [ None, {
+    'AddressFamily' : [ 0x18, ['unsigned short']],
+    }],
+    '_LOCAL_ADDRESS_WIN10_UDP' : [ None, {
+    'pData' : [ 0x0, ['pointer', ['_IN_ADDR']]],
+    }],
+    '_LOCAL_ADDRESS' : [ None, {
+    'pData' : [ 0x10, ['pointer', ['pointer', ['_IN_ADDR']]]],
+    }],
+    '_ADDRINFO' : [ None, {
+    'Local' : [ 0x0, ['pointer', ['_LOCAL_ADDRESS']]],
+    'Remote' : [ 0x10, ['pointer', ['_IN_ADDR']]],
+    }],
+    '_TCP_LISTENER': [ None, { # TcpL
+    'Owner' : [ 0x30, ['pointer', ['_EPROCESS']]],
+    'CreateTime' : [ 0x40, ['WinTimeStamp', dict(is_utc = True)]],
+    'LocalAddr' : [ 0x60, ['pointer', ['_LOCAL_ADDRESS']]],
+    'InetAF' : [ 0x28, ['pointer', ['_INETAF']]],
+    'Port' : [ 0x72, ['unsigned be short']],
+    }],
+    '_TCP_ENDPOINT': [ None, { # TcpE
+    'InetAF' : [ 0x10, ['pointer', ['_INETAF']]],
+    'AddrInfo' : [ 0x18, ['pointer', ['_ADDRINFO']]],
+    'State' : [ 0x6C, ['Enumeration', dict(target = 'long', choices = TCP_STATE_ENUM)]],
+    'LocalPort' : [ 0x70, ['unsigned be short']],
+    'RemotePort' : [ 0x72, ['unsigned be short']],
+    'Owner' : [ 0x258, ['pointer', ['_EPROCESS']]],
+    'CreateTime' : [ 0x268, ['WinTimeStamp', dict(is_utc = True)]],
+    }],
+    '_UDP_ENDPOINT': [ None, { # UdpA
+    'Owner' : [ 0x28, ['pointer', ['_EPROCESS']]],
+    'CreateTime' : [ 0x58, ['WinTimeStamp', dict(is_utc = True)]],
+    'LocalAddr' : [ 0x80, ['pointer', ['_LOCAL_ADDRESS_WIN10_UDP']]],
+    'InetAF' : [ 0x20, ['pointer', ['_INETAF']]],
+    'Port' : [ 0x78, ['unsigned be short']],
+    }],
+}
+
+
+class _ADDRESS_OBJECT(obj.CType):
+
+    def is_valid(self):
+        return obj.CType.is_valid(self) and self.CreateTime.v() > 0
+
+class WinXP2003AddressObject(obj.ProfileModification):
+    conditions = {'os': lambda x: x == 'windows',
+                  'major': lambda x : x == 5}
+    def modification(self, profile):
+        profile.object_classes.update({'_ADDRESS_OBJECT': _ADDRESS_OBJECT})
+
 class WinXP2003Tcpipx64(obj.ProfileModification):
     before = ['WindowsVTypes']
     conditions = {'os': lambda x: x == 'windows',
@@ -330,3 +385,133 @@ class Win7x64Tcpip(obj.ProfileModification):
                 'RemoteAddress' : [ 0x58, ['pointer', ['_IN_ADDR']]],
                 }], 
             })
+
+class Win8Tcpip(obj.ProfileModification):
+    before = ['Vista2008Tcpip']
+    conditions = {'os': lambda x: x == 'windows',
+                  'memory_model': lambda x: x == '32bit',
+                  'major': lambda x : x == 6,
+                  'minor': lambda x : x >= 2}
+    def modification(self, profile):
+        profile.merge_overlay({
+        '_TCP_ENDPOINT': [ None, {
+            'InetAF' : [ 0x8, ['pointer', ['_INETAF']]],
+            'AddrInfo' : [ 0xC, ['pointer', ['_ADDRINFO']]],
+            'State' : [ 0x38, ['Enumeration', dict(target = 'long', choices = TCP_STATE_ENUM)]],
+            'LocalPort' : [ 0x3C, ['unsigned be short']],
+            'RemotePort' : [ 0x3E, ['unsigned be short']],
+            'Owner' : [ 0x174, ['pointer', ['_EPROCESS']]],
+            }],
+        '_ADDRINFO' : [ None, {
+            'Remote' : [ 0xC, ['pointer', ['_IN_ADDR']]],
+            }],
+        })
+
+class Win81Tcpip(obj.ProfileModification):
+    before = ['Win8Tcpip']
+    conditions = {'os': lambda x: x == 'windows',
+                  'memory_model': lambda x: x == '32bit',
+                  'major': lambda x : x == 6,
+                  'minor': lambda x : x == 3}
+    def modification(self, profile):
+        profile.merge_overlay({
+        '_TCP_ENDPOINT': [ None, {
+            'Owner' : [ 0x1a8, ['pointer', ['_EPROCESS']]],
+            }],
+        })
+
+class Win10Tcpip(obj.ProfileModification):
+    before = ['Win8Tcpip']
+    conditions = {'os': lambda x: x == 'windows',
+                  'memory_model': lambda x: x == '32bit',
+                  'major': lambda x : x == 6,
+                  'minor': lambda x : x >= 4}
+    def modification(self, profile):
+        profile.merge_overlay({
+        '_ADDRINFO' : [ None, {
+            'Local' : [ 0x0, ['pointer', ['_LOCAL_ADDRESS']]],
+            'Remote' : [ 0xC, ['pointer', ['_IN_ADDR']]],
+        }],
+        '_TCP_ENDPOINT': [ None, {
+            'InetAF' : [ 0x8, ['pointer', ['_INETAF']]],
+            'AddrInfo' : [ 0xC, ['pointer', ['_ADDRINFO']]],
+            'State' : [ 0x38, ['Enumeration', dict(target = 'long', choices = TCP_STATE_ENUM)]],
+            'LocalPort' : [ 0x3C, ['unsigned be short']],
+            'RemotePort' : [ 0x3E, ['unsigned be short']],
+            'Owner' : [ 0x1b0, ['pointer', ['_EPROCESS']]],
+            }],
+        })
+
+        build = profile.metadata.get("build")
+
+        if build == 14393:
+            profile.merge_overlay({
+                '_TCP_ENDPOINT': [ None, {
+                'Owner' : [ 0x1b4, ['pointer', ['_EPROCESS']]],
+                }],
+            })
+        elif build >= 15063:
+            profile.merge_overlay({
+                '_TCP_ENDPOINT': [ None, {
+                'Owner' : [ 0x1cc, ['pointer', ['_EPROCESS']]],
+                }],
+            })
+
+class Win8x64Tcpip(obj.ProfileModification):
+    before = ['Win7Vista2008x64Tcpip']
+    conditions = {'os': lambda x: x == 'windows',
+                  'memory_model': lambda x: x == '64bit',
+                  'major': lambda x : x == 6,
+                  'minor': lambda x : x >= 2}
+    def modification(self, profile):
+        profile.merge_overlay({
+            '_INETAF' : [ None, {
+                'AddressFamily' : [ 0x18, ['unsigned short']],
+                }],
+            '_TCP_ENDPOINT': [ None, {
+                'InetAF' : [ 0x10, ['pointer', ['_INETAF']]],
+                'AddrInfo' : [ 0x18, ['pointer', ['_ADDRINFO']]],
+                'State' : [ 0x6C, ['Enumeration', dict(target = 'long', choices = TCP_STATE_ENUM)]],
+                'LocalPort' : [ 0x70, ['unsigned be short']],
+                'RemotePort' : [ 0x72, ['unsigned be short']],
+                'Owner' : [ 0x250, ['pointer', ['_EPROCESS']]],
+                }],
+            })
+
+class Win81x64Tcpip(obj.ProfileModification):
+    before = ['Win8x64Tcpip']
+    conditions = {'os': lambda x: x == 'windows',
+                  'memory_model': lambda x: x == '64bit',
+                  'major': lambda x : x == 6,
+                  'minor': lambda x : x == 3}
+    def modification(self, profile):
+        profile.merge_overlay({
+            '_TCP_ENDPOINT': [ None, {
+                'Owner' : [ 0x258, ['pointer', ['_EPROCESS']]],
+                }],
+            })
+
+class Win10x64Tcpip(obj.ProfileModification):
+    before = ['Win81x64Tcpip']
+    conditions = {'os': lambda x: x == 'windows',
+                  'memory_model': lambda x: x == '64bit',
+                  'major': lambda x : x == 6,
+                  'minor': lambda x : x == 4}
+    def modification(self, profile):
+        profile.vtypes.update(tcpip_vtypes_win_10_x64)
+        
+class Win10x64_15063_Tcpip(obj.ProfileModification):
+    """TCP Endpoint for Creators and Fall Creators"""
+    
+    before = ['Win10x64Tcpip']
+    conditions = {'os': lambda x: x == 'windows',
+                  'memory_model': lambda x: x == '64bit',
+                  'major': lambda x : x == 6,
+                  'minor': lambda x : x == 4,
+                  'build': lambda x : x >= 15063}
+    def modification(self, profile):
+        profile.merge_overlay({
+            '_TCP_ENDPOINT': [ None, {
+                'Owner' : [ 0x270, ['pointer', ['_EPROCESS']]],
+                }],
+            })
