@@ -34,10 +34,12 @@ import volatility.utils as utils
 import volatility.plugins.malware.apihooks as apihooks
 import volatility.debug as debug #pylint: disable-msg=W0611
 from volatility.cache import CacheDecorator
+from volatility.renderers import TreeGrid
+from volatility.renderers.basic import Address
 
 #pylint: disable-msg=C0111
 
-def find_tables(start_addr, vm):
+def find_tables(nt_base, start_addr, vm):
     """
     This function finds the RVAs to KeServiceDescriptorTable
     and KeServiceDescriptorTableShadow in the NT module. 
@@ -79,12 +81,15 @@ def find_tables(start_addr, vm):
             if op.flowControl == 'FC_RET':
                 break
             # Looking for a 9-byte CMP instruction whose first operand
-            # has a 32-bit displacement and second operand is zero 
-            if op.mnemonic == 'CMP' and op.size == 9 and op.operands[0].dispSize == 32 and op.operands[0].value == 0:
-                # The displacement is the RVA we want 
-                service_tables.append(op.operands[0].disp)
+            # has a 32-bit displacement and second operand is zero or
+            # a 7-byte CMP instruction with RIP relative addressing
+            if op.mnemonic == 'CMP' and op.operands[0].dispSize == 32 and op.operands[0].value == 0:
+                if op.size == 9:
+                    service_tables.append(nt_base + op.operands[0].disp)
+                elif op.size in [7, 8]:
+                    service_tables.append(op.address + op.size + op.operands[0].disp)
             elif op.mnemonic == 'LEA' and op.size == 7 and op.operands[1].dispSize == 32 and op.operands[1].disp > 0:
-                service_tables.append(op.operands[1].disp)
+                service_tables.append(nt_base + op.operands[1].disp)
     else:
         vm.profile.add_types({
             '_INSTRUCTION' : [ 9, {
@@ -107,7 +112,7 @@ def find_tables(start_addr, vm):
             if op.value == 0:
                 for s in ops_list:
                     if op.opcode.v().startswith(s):
-                        service_tables.append(op.disp)
+                        service_tables.append(nt_base + op.disp)
 
     return service_tables
 
@@ -148,8 +153,8 @@ class SSDT(common.AbstractWindowsCommand):
             if func_rva == None:
                 raise StopIteration("Cannot locate KeAddSystemServiceTable")
             KeAddSystemServiceTable = ntos.DllBase + func_rva
-            for table_rva in find_tables(KeAddSystemServiceTable, addr_space):
-                ssdt_obj = obj.Object("_SERVICE_DESCRIPTOR_TABLE", ntos.DllBase + table_rva, addr_space)
+            for table_addr in find_tables(ntos.DllBase, KeAddSystemServiceTable, addr_space):
+                ssdt_obj = obj.Object("_SERVICE_DESCRIPTOR_TABLE", table_addr, addr_space)
                 ssdts.add(ssdt_obj)
 
         # Get a list of *unique* SSDT entries. Typically we see only two.
@@ -159,7 +164,9 @@ class SSDT(common.AbstractWindowsCommand):
             for i, desc in enumerate(ssdt_obj.Descriptors):
                 # Apply some extra checks - KiServiceTable should reside in kernel memory and ServiceLimit 
                 # should be greater than 0 but not unbelievably high
-                if desc.is_valid() and desc.ServiceLimit > 0 and desc.ServiceLimit < 0xFFFF and desc.KiServiceTable > 0x80000000:
+                if not desc.is_valid() or desc.ServiceLimit <= 0 or desc.ServiceLimit >= 2048 or desc.KiServiceTable <= 0x80000000:
+                    break
+                else:
                     tables.add((i, desc.KiServiceTable.v(), desc.ServiceLimit.v()))
 
         print "Finding appropriate address space for tables..."
@@ -175,8 +182,105 @@ class SSDT(common.AbstractWindowsCommand):
         for idx, table, n, vm in sorted(tables_with_vm, key = itemgetter(0)):
             yield idx, table, n, vm, mods, mod_addrs
 
-    def render_text(self, outfd, data):
+    def unified_output(self, data):
+        if not self._config.VERBOSE:
+            return TreeGrid([("Table", str),
+                       ("TableOffset", Address),
+                       ("NumEntries", int),
+                       ("Entry", Address),
+                       ("Addr", Address),
+                       ("Function", str),
+                       ("Owner", str)],
+                        self.generator(data))
+        else:
+            return TreeGrid([("Table", str),
+                       ("TableOffset", Address),
+                       ("NumEntries", int),
+                       ("Entry", Address),
+                       ("Addr", Address),
+                       ("Function", str),
+                       ("Owner", str),
+                       ("Destination", Address),
+                       ("HookName", str)],
+                        self.generator(data))
 
+
+    def generator(self, data):
+
+        addr_space = utils.load_as(self._config)
+        syscalls = addr_space.profile.syscalls
+        bits32 = addr_space.profile.metadata.get('memory_model', '32bit') == '32bit'
+
+        # Print out the entries for each table
+        for idx, table, n, vm, mods, mod_addrs in data:
+            table_name = "SSDT[{0}]".format(idx)
+            table_offset = Address(table)
+            num_entries = int(n)
+            for i in range(n):
+                if bits32:
+                    # These are absolute function addresses in kernel memory. 
+                    syscall_addr = obj.Object('address', table + (i * 4), vm).v()
+                else:
+                    # These must be signed long for x64 because they are RVAs relative
+                    # to the base of the table and can be negative. 
+                    offset = obj.Object('long', table + (i * 4), vm).v()
+                    if offset == None:
+                        continue
+
+                    # The offset is the top 20 bits of the 32 bit number. 
+                    syscall_addr = table + (offset >> 4)
+                try:
+                    syscall_name = syscalls[idx][i]
+                except IndexError:
+                    syscall_name = "UNKNOWN"
+
+                syscall_mod = tasks.find_module(mods, mod_addrs, addr_space.address_mask(syscall_addr))
+                if syscall_mod:
+                    syscall_modname = syscall_mod.BaseDllName
+                else:
+                    syscall_modname = "UNKNOWN"
+
+                if not self._config.VERBOSE:
+                    yield (0, [table_name, table_offset, num_entries, Address(idx * 0x1000 + i), 
+                            Address(syscall_addr), str(syscall_name), str(syscall_modname)])
+
+                ## check for inline hooks if in --verbose mode, we're analyzing
+                ## an x86 model system and the sycall_mod is available 
+                if (self._config.VERBOSE and 
+                            addr_space.profile.metadata.get('memory_model', '32bit') == '32bit' and 
+                            syscall_mod is not None):
+
+                        ## leverage this static method from apihooks
+                        ret = apihooks.ApiHooks.check_inline(va = syscall_addr, addr_space = vm, 
+                                                mem_start = syscall_mod.DllBase, 
+                                                mem_end = syscall_mod.DllBase + syscall_mod.SizeOfImage)
+                        ## could not analyze the memory
+                        if ret == None:
+                            yield (0, [table_name, table_offset, num_entries, Address(idx * 0x1000 + i), 
+                                    Address(syscall_addr), str(syscall_name), str(syscall_modname), 
+                                    Address(0), "NotInline"])
+                            continue 
+                        (hooked, data, dest_addr) = ret
+                        ## the function isn't hooked
+                        if not hooked:
+                            yield (0, [table_name, table_offset, num_entries, Address(idx * 0x1000 + i), 
+                                    Address(syscall_addr), str(syscall_name), str(syscall_modname), 
+                                    Address(0), "NotInline"])
+                            continue 
+                        ## we found a hook, try to resolve the hooker. no mask required because
+                        ## we currently only work on x86 anyway
+                        hook_mod = tasks.find_module(mods, mod_addrs, dest_addr)
+                        if hook_mod: 
+                            hook_name = hook_mod.BaseDllName
+                        else:
+                            hook_name = "UNKNOWN"
+                        ## report it now 
+                        yield (0, [table_name, table_offset, num_entries, Address(idx * 0x1000 + i), 
+                                Address(syscall_addr), str(syscall_name), str(syscall_modname),
+                                Address(dest_addr), str(hook_name)])
+
+
+    def render_text(self, outfd, data):
         addr_space = utils.load_as(self._config)
         syscalls = addr_space.profile.syscalls
         bits32 = addr_space.profile.metadata.get('memory_model', '32bit') == '32bit'
@@ -192,6 +296,9 @@ class SSDT(common.AbstractWindowsCommand):
                     # These must be signed long for x64 because they are RVAs relative
                     # to the base of the table and can be negative. 
                     offset = obj.Object('long', table + (i * 4), vm).v()
+                    if offset == None:
+                        continue
+
                     # The offset is the top 20 bits of the 32 bit number. 
                     syscall_addr = table + (offset >> 4)
                 try:
@@ -236,4 +343,3 @@ class SSDT(common.AbstractWindowsCommand):
                             hook_name = "UNKNOWN"
                         ## report it now 
                         outfd.write("  ** INLINE HOOK? => {0:#x} ({1})\n".format(dest_addr, hook_name))
-
